@@ -3,18 +3,21 @@ use 5.008_001;
 use strict;
 use warnings;
 use Carp;
+use Data::Recursive::Encode;
+use Lingua::JA::Regular::Unicode qw/alnum_h2z space_h2z katakana_h2z/;
 
-our $VERSION = '0.23';
-
+our $VERSION = '0.30';
 
 require XSLoader;
 XSLoader::load(__PACKAGE__, $VERSION);
 
+
 sub _options
 {
     return {
         # analysis options
         model   => '/usr/local/share/kytea/model.bin',
+        h2z     => 1,
         nows    => 0,
         notags  => 0,
         notag   => [],
@@ -55,6 +58,36 @@ sub new
     return _init_text_kytea($class, $options);
 }
 
+sub _h2z { katakana_h2z( space_h2z( alnum_h2z($_[0]) ) ); }
+
+sub parse
+{
+    my ($self, $text) = @_;
+
+    my $is_h2z_enable = $self->_is_h2z_enable;
+
+    if ($is_h2z_enable)
+    {
+        my @original_chars = split(//, $text);
+        my $text = _h2z($text);
+
+        my $results = Data::Recursive::Encode->decode_utf8( $self->_parse($text) );
+
+        my $i = 0;
+
+        # changed char -> original char
+        for my $result (@{$results})
+        {
+            $result->{surface} = join( '', @original_chars[$i .. $i + (length $result->{surface}) - 1] );
+            $i += length $result->{surface};
+        }
+
+        return $results;
+    }
+
+    return Data::Recursive::Encode->decode_utf8( $self->_parse($text) );
+}
+
 1;
 __END__
 
@@ -70,6 +103,7 @@ my ($text, %config, $path);
 =head1 SYNOPSIS
 
   use Text::KyTea;
+  use utf8;
 
   my $kytea   = Text::KyTea->new(%config);
   my $results = $kytea->parse($text);
@@ -98,14 +132,14 @@ KyTea is a general toolkit developed for analyzing text,
 with a focus on Japanese, Chinese and other languages
 requiring word or morpheme segmentation.
 
-This module works under KyTea Ver.0.3.2 or later.
-Under old versions of KyTea, this might not works.
+This module works under KyTea Ver.0.3.2 and later.
+Under old versions of KyTea, this might not work.
 
 If you've changed default install directory of KyTea,
 please install Text::KyTea with interactive mode
-(e.g. cpanm --interactive or cpanm -v).
+(e.g., cpanm --interactive or cpanm -v).
 
-For more information about KyTea, please see the SEE ALSO.
+For more information about KyTea, please see the "SEE ALSO" section.
 
 
 =head1 METHODS
@@ -118,6 +152,7 @@ Creates a new Text::KyTea instance.
 
   my $kytea = Text::KyTea->new(
       model   => 'model.bin', # default is '/usr/local/share/kytea/model.bin'
+      h2z     => 0,           # default is 1 (enable)
       notag   => [1,2],       # default is []
       nounk   => 0,           # default is 0 (estimates the pronunciation of unkown words)
       unkbeam => 50,          # default is 50
@@ -127,6 +162,12 @@ Creates a new Text::KyTea instance.
   );
 
 
+=item new(h2z => 1)
+
+Converts $text from hankaku to zenkaku before parsing $text.
+This option improves the parsing accuracy in most of model files.
+
+
 =item read_model($path)
 
 Reads the given model file.
@@ -141,9 +182,6 @@ Parses the given text via KyTea, and returns results of analysis.
 The results are returned as an array reference.
 
 
-=item write_model($path)
-
-
 =back
 
 =head1 AUTHOR
@@ -156,7 +194,7 @@ http://www.phontron.com/kytea/
 
 =head1 LICENSE
 
-Copyright (C) 2011 pawa All rights reserved.
+Copyright (C) 2012 pawa All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
